server: Introduce command_{pre,post}_select().
[paraslash.git] / signal.c
index 78e788d19021111b2f8ec712eb1ff8f0fd176375..f8be2f4612002214f669fa74ce92390866b0dc90 100644 (file)
--- a/signal.c
+++ b/signal.c
@@ -73,7 +73,7 @@ static void generic_signal_handler(int s)
  * one otherwise. If and only if the function returns one, the content of \a
  * pid is meaningful.
  *
- * \sa waitpid(2)
+ * \sa waitpid(2).
  */
 int para_reap_child(pid_t *pid)
 {
@@ -110,12 +110,15 @@ void para_reap_children(void)
 }
 
 /**
- * wrapper around signal(2)
- * \param sig the number of the signal to catch
+ * Wrapper around signal(2).
+ *
+ * \param sig The number of the signal to catch.
  *
  * This installs the generic signal handler for the given signal.
+ *
  * \return This function returns 1 on success and \p -E_SIGNAL_SIG_ERR on errors.
- * \sa signal(2)
+ *
+ * \sa signal(2).
  */
 int para_install_sighandler(int sig)
 {
@@ -124,29 +127,35 @@ int para_install_sighandler(int sig)
 }
 
 /**
- * return number of next pending signal
+ * Return the number of next pending signal.
  *
  * This should be called if the fd for the signal pipe is ready for reading.
  *
* \return On success, the number of the received signal is returned. \p
* -E_SIGNAL_READ is returned if a read error occurred while reading the signal
* pipe.  If the read was interrupted by another signal the function returns 0.
  */
 int para_next_signal(void)
 {
        int s;
-       ssize_t r;
+       ssize_t r = read(signal_pipe[0], &s, sizeof(s));
 
-       r = read(signal_pipe[0], &s, sizeof(s));
-       if (r == sizeof(s)) {
-               PARA_DEBUG_LOG("next signal: %d\n", s);
-               return s;
+       if (!r) {
+               PARA_CRIT_LOG("read from signal pipe returned zero\n");
+               return 0;
+       }
+       if (r < 0) {
+               if (errno == EAGAIN || errno == EINTR)
+                       return 0;
+               return -ERRNO_TO_PARA_ERROR(errno);
        }
-       return r < 0 && (errno != EAGAIN)? 0 : -E_SIGNAL_READ;
+       assert(r == sizeof(s));
+       PARA_DEBUG_LOG("next signal: %d\n", s);
+       return s;
 }
 
 /**
- * Close the signal pipe.
+ * Close the write end of the signal pipe.
  */
 void para_signal_shutdown(void)
 {
@@ -124,29 +127,35 @@ int para_install_sighandler(int sig)
  * This should be called if the fd for the signal pipe is ready for reading.
  *
- * \return On success, the number of the received signal is returned. \p
- * -E_SIGNAL_READ is returned if a read error occurred while reading the signal
- * pipe.  If the read was interrupted by another signal the function returns 0.
+ * \return On success, the number of the received signal is returned.  If the
+ * read returned zero or was interrupted by another signal the function returns
+ * 0.  Otherwise, a negative error value is returned.
  */
 int para_next_signal(void)